=== modified file 'NEWS'
--- NEWS	2009-09-07 03:00:23 +0000
+++ NEWS	2009-09-07 23:47:14 +0000
@@ -9,6 +9,13 @@
 bzr 2.0rc2 (not released yet)
 #############################
 
+New Features
+************
+
+* Added post_commit hook for mutable trees. This allows the keywords
+  plugin to expand keywords on files changed by the commit.
+  (Ian Clatworthy, #408841)
+
 Bug Fixes
 *********
 

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2009-07-10 08:33:11 +0000
+++ bzrlib/mutabletree.py	2009-09-07 23:14:05 +0000
@@ -226,6 +226,9 @@
             revprops=revprops,
             possible_master_transports=possible_master_transports,
             *args, **kwargs)
+        post_hook_params = PostCommitHookParams(self)
+        for hook in MutableTree.hooks['post_commit']:
+            hook(post_hook_params)
         return committed_id
 
     def _gather_kinds(self, files, kinds):
@@ -581,14 +584,32 @@
         self.create_hook(hooks.HookPoint('start_commit',
             "Called before a commit is performed on a tree. The start commit "
             "hook is able to change the tree before the commit takes place. "
-            "start_commit is called with the bzrlib.tree.MutableTree that the "
-            "commit is being performed on.", (1, 4), None))
+            "start_commit is called with the bzrlib.mutabletree.MutableTree "
+            "that the commit is being performed on.", (1, 4), None))
+        self.create_hook(hooks.HookPoint('post_commit',
+            "Called after a commit is performed on a tree. The hook is "
+            "called with a bzrlib.mutabletree.PostCommitHookParams object. "
+            "The mutable tree the commit was performed on is available via "
+            "the mutable_tree attribute of that object.", (2, 0), None))
 
 
 # install the default hooks into the MutableTree class.
 MutableTree.hooks = MutableTreeHooks()
 
 
+class PostCommitHookParams(object):
+    """Parameters for the post_commit hook.
+
+    To access the parameters, use the following attributes:
+
+    * mutable_tree - the MutableTree object
+    """
+
+    def __init__(self, mutable_tree):
+        """Create the parameters for the post_commit hook."""
+        self.mutable_tree = mutable_tree
+
+
 class _FastPath(object):
     """A path object with fast accessors for things like basename."""
 

=== modified file 'bzrlib/tests/per_workingtree/test_commit.py'
--- bzrlib/tests/per_workingtree/test_commit.py	2009-08-04 11:40:59 +0000
+++ bzrlib/tests/per_workingtree/test_commit.py	2009-09-07 23:14:05 +0000
@@ -611,3 +611,26 @@
         revid = tree.commit('first post')
         committed_tree = tree.basis_tree()
         self.assertTrue(committed_tree.has_filename("newfile"))
+
+    def test_post_commit_hook(self):
+        """Make sure a post_commit hook is called after a commit."""
+        def post_commit_hook_test_params(params):
+            self.assertTrue(isinstance(params,
+                mutabletree.PostCommitHookParams))
+            self.assertTrue(isinstance(params.mutable_tree,
+                mutabletree.MutableTree))
+            open(tree.abspath("newfile"), 'w').write("data")
+            params.mutable_tree.add(["newfile"])
+        def restoreDefaults():
+            mutabletree.MutableTree.hooks['post_commit'] = []
+        self.addCleanup(restoreDefaults)
+        tree = self.make_branch_and_tree('.')
+        mutabletree.MutableTree.hooks.install_named_hook(
+            'post_commit',
+            post_commit_hook_test_params,
+            None)
+        self.assertFalse(tree.has_filename("newfile"))
+        revid = tree.commit('first post')
+        self.assertTrue(tree.has_filename("newfile"))
+        committed_tree = tree.basis_tree()
+        self.assertFalse(committed_tree.has_filename("newfile"))

=== modified file 'bzrlib/tests/test_mutabletree.py'
--- bzrlib/tests/test_mutabletree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_mutabletree.py	2009-09-07 23:14:05 +0000
@@ -30,6 +30,8 @@
         hooks = MutableTreeHooks()
         self.assertTrue("start_commit" in hooks,
                         "start_commit not in %s" % hooks)
+        self.assertTrue("post_commit" in hooks,
+                        "post_commit not in %s" % hooks)
 
     def test_installed_hooks_are_MutableTreeHooks(self):
         """The installed hooks object should be a MutableTreeHooks."""

